/* Copyright (c) 1999 Christopher Y. Wong
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class DocReaderCtrl implements ActionListener,
  ComponentListener, KeyListener, AdjustmentListener
{
  public DocReaderCtrl()
  {
    view = new DocReaderView(this);
    model = new DocReaderModel(view.getViewBounds());
    model.addObserver(view);
    searcher = new Searcher(view, model);
    bookmarker = new BookmarkCtrl(view, model);

    fileDialog = new FileDialog(view, "Open Doc file", FileDialog.LOAD);  // !!!
    //fileDialog.setTitle("Open Doc file");
    //fileDialog.setMode(FileDialog.LOAD);
  }

   public void run(String path)
   {
      view.setVisible(true);
      if (path != null)
	 model.open(path);
   }

  public void actionPerformed(ActionEvent e)
  {
    Object source = e.getSource();

    if (source == view.openButton)
      openFile();
    else if (source == view.searchButton)
      searcher.go();
    else if (source == view.bmkButton)
      bookmarker.go();
    else if (source == view.topButton)
      model.goTop();
    else if (source == view.bottomButton)
      model.goBottom();
    else if (source == view.prefsButton) { // !!!
      PrefsView prefsView = new PrefsView(view, this);
      prefsView.show();
    };
    /*
    else if (source == view.quitButton)
    BookmarkModel bookmarks = model.getBookmarks();
    if (bookmarks != null)
      bookmarks.save();
    System.exit(0);
    */
  }

  public void setDirectory(String dir) {  // !!!
//System.out.println("setDirectory: " + dir);
    fileDialog.setDirectory(dir + "Documents/");
  }

  public void setFont(String name, int size) {  // !!!
//System.out.println("setFont: " + name + ' ' + size);
    Config.init(name, size);
    view.textArea.setFont(Config.getTextAreaFont());
    (view.getViewBounds()).setBounds(view.textArea, true);
    model.loadPage();
    //view.textArea.repaint();
  }

  private void openFile()
  {
    //fileDialog.setFile("*.p??");
    fileDialog.setFilenameFilter(new PDB_PRC_Filter());
    fileDialog.show();
    if (fileDialog.getFile() != null) {
      model.open(fileDialog.getDirectory() + fileDialog.getFile());
    };
  }

  public void componentResized(ComponentEvent e)
  {
    view.adjustBounds();
    model.loadPage();
  }

   public void keyPressed(KeyEvent e)
   {
      switch (e.getKeyCode()) {
      case KeyEvent.VK_DOWN:
	model.lineDown();
	e.consume();
	break;
      case KeyEvent.VK_PAGE_DOWN:
	model.pageDown();
	e.consume();
	break;
      case KeyEvent.VK_UP:
	model.lineUp();
	e.consume();
	break;
      case KeyEvent.VK_PAGE_UP:
	model.pageUp();
	e.consume();
	break;
      case KeyEvent.VK_HOME:
	model.goTop();
	e.consume();
	break;
      case KeyEvent.VK_END:
	model.goBottom();
	e.consume();
	break;
      }
   }

   public void adjustmentValueChanged(AdjustmentEvent e)
   {
      if (!model.isValid()) return;

      switch (e.getAdjustmentType()) {
      case AdjustmentEvent.UNIT_INCREMENT:
	 model.lineDown();
	 break;
      case AdjustmentEvent.UNIT_DECREMENT:
	 model.lineUp();
	 break;
      case AdjustmentEvent.BLOCK_INCREMENT:
	 model.pageDown();
	 break;
      case AdjustmentEvent.BLOCK_DECREMENT:
	 model.pageUp();
	 break;
      case AdjustmentEvent.TRACK:
	 int newLoc = e.getValue();
	 if (0 <= newLoc && newLoc <= model.getEndLoc())
	    model.setCurrentLoc(newLoc);
      default:
      }
   }

   public void keyReleased(KeyEvent e) {}
   public void keyTyped(KeyEvent e) {}

  public void componentMoved(ComponentEvent e) {}
  public void componentShown(ComponentEvent e) {}
  public void componentHidden(ComponentEvent e) {}

  DocReaderView view;
  DocReaderModel model;
  FileDialog fileDialog;
  Searcher searcher;
  BookmarkCtrl bookmarker;
}

// !!!
class PDB_PRC_Filter
  implements FilenameFilter {

  PDB_PRC_Filter() {
  }

  public boolean accept(File dir, String name) {
    return (name.endsWith(".pdb") || name.endsWith(".prc"));
  }
}

// !!! use ...Model, Ctrl ?; save?
class PrefsView
  extends Dialog
  implements
    ItemListener,     // for Choice
    ActionListener    // for Button
  {

  Button ramButton, sdButton, cfButton;
  Choice fontList, sizeList;
  DocReaderCtrl ctrl;
  private static final String[] FONT_SIZES = {"10", "12", "14", "16", "18"};

  public void addChoiceItems(Choice choice, String[] items) {
    for (int i=0; i < items.length; i++)
      choice.add(items[i]);
  }

  PrefsView(Frame parent, DocReaderCtrl controller) {
    super(parent, "Prefs: DocDir, Font", false);
    ctrl = controller;

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent ev) {
        dispose();
      }
    });

    Panel p = new Panel();
    p.add(new Label("Document Directory:"));
    ramButton = new Button("RAM");
    ramButton.addActionListener(this);
    p.add(ramButton);

    cfButton = new Button("CF");
    cfButton.addActionListener(this);
    p.add(cfButton);

    sdButton = new Button("SD");
    sdButton.addActionListener(this);
    p.add(sdButton);
    add(p, BorderLayout.NORTH);

    p = new Panel();
    p.add(new Label("Default Font:"));
    Font configFont = Config.getTextAreaFont();
    fontList = new Choice();
    // getFontList deprecated in 1.2
    // but GraphicsEnvironment.getAvailableFontFamilyNames() not avail in 1.1
    addChoiceItems(fontList, Toolkit.getDefaultToolkit().getFontList());
    fontList.select(configFont.getName());
    fontList.addItemListener(this);
    p.add(fontList);

    sizeList = new Choice();
    addChoiceItems(sizeList, FONT_SIZES);
    sizeList.select(Integer.toString(configFont.getSize()));
    sizeList.addItemListener(this);
    p.add(sizeList);
    add(p, BorderLayout.SOUTH);
    pack();
  }

  public void actionPerformed(ActionEvent ev) {
    Object obj = ev.getSource();
    if (obj == ramButton)
      ctrl.setDirectory("/home/root/");
    else if (obj == sdButton)
      ctrl.setDirectory("/mnt/card/");
    else if (obj == cfButton)
      ctrl.setDirectory("/mnt/cf/");
  }

  public void itemStateChanged(ItemEvent ev) {
    ctrl.setFont(
      fontList.getSelectedItem(),
      Integer.parseInt(sizeList.getSelectedItem())
      );
  }
}
